/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef ASM_ARM_BOOT_H
#define ASM_ARM_BOOT_H

#define ELF_CLASS	ELFCLASS32
#define ELF_DATA	ELFDATA2LSB
#define ELF_ARCH	EM_ARM

#endif /* ASM_ARM_BOOT_H */
