/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _AMD_00730F01_PCI_DEVS_H_
#define _AMD_00730F01_PCI_DEVS_H_

#define BUS0 0

/* Graphics and Display */
#define GFX_DEV 0x1
#define GFX_FUNC 0
#define GFX_DEVID 0x9850
#define GFX_DEVFN PCI_DEVFN(GFX_DEV,GFX_FUNC)

/* Internal Audio controller */
#define ACTL_DEV 0x1
#define ACTL_FUNC 1
#define ACTL_DEVID 0x9840
#define ACTL_DEVFN PCI_DEVFN(ACTL_DEV,ACTL_FUNC)

/* PCIe Ports */
#define NB_PCIE_PORT2_DEV 0x2
#define NB_PCIE_FUNC0	0
#define NB_PCIE_FUNC1	1
#define NB_PCIE_FUNC2	2
#define NB_PCIE_FUNC3	3
#define NB_PCIE_FUNC4	4
#define NB_PCIE_FUNC5	5
#define NB_PCIE_PORT_DEVID 0x1439
#define NB_PCIE_PORT0_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC0)
#define NB_PCIE_PORT1_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC1)
#define NB_PCIE_PORT2_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC2)
#define NB_PCIE_PORT3_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC3)
#define NB_PCIE_PORT4_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC4)
#define NB_PCIE_PORT5_DEVFN PCI_DEVFN(NB_PCIE_PORT2_DEV,NB_PCIE_FUNC5)

#endif /* _AMD_00730F01_PCI_DEVS_H_ */
