/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef RAM_SEGS_H
#define RAM_SEGS_H

#define RAM_CODE_SEG            0x10
#define RAM_DATA_SEG            0x18
#define RAM_CODE16_SEG          0x28
#define RAM_DATA16_SEG          0x30
#define RAM_CODE_ACPI_SEG       0x38
#define RAM_DATA_ACPI_SEG       0x40
#define RAM_CODE_SEG64          0x48

#endif /* RAM_SEGS_H */
