/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _EC_QUANTA_ENE_KB3940Q_CHIP_H
#define _EC_QUANTA_ENE_KB3940Q_CHIP_H

struct ec_quanta_ene_kb3940q_config {
};

#endif /* _EC_QUANTA_ENE_KB3940Q_CHIP_H */
