/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __I2C_GPIOMUX_BUS_CHIP_H__
#define __I2C_GPIOMUX_BUS_CHIP_H__

struct drivers_i2c_gpiomux_bus_config {
};

#endif /* __I2C_GPIOMUX_BUS_CHIP_H__ */
