/** @file
  This file contains the latest ACPI definitions that are
  consumed by drivers that do not care about ACPI versions.

  Copyright (c) 2006 - 2016, Intel Corporation. All rights reserved.<BR>
  This file and the accompanying materials are licensed and made available under
  the terms and conditions of the BSD License.
  The full text of the license may be found at
  http://opensource.org/licenses/bsd-license.php

  THIS FILE IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.

**/

#ifndef _ACPI_H_
#define _ACPI_H_

#include <IndustryStandard/Acpi60.h>

#endif
