/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef EC_GOOGLE_WILCO_ROMSTAGE_H
#define EC_GOOGLE_WILCO_ROMSTAGE_H

/**
 * wilco_ec_early_init
 *
 * This function performs initialization of the EC in romstage.
 */
void wilco_ec_romstage_init(void);

#endif /* EC_GOOGLE_WILCO_ROMSTAGE_H */
