/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef EC_LENOVO_PMH7_CHIP_H
#define EC_LENOVO_PMH7_CHIP_H

struct ec_lenovo_pmh7_config {
	int backlight_enable:1;
	int dock_event_enable:1;
};

#endif /* EC_LENOVO_PMH7_CHIP_H */
