#ifndef DEVICE_PNP_DEF_H
#define DEVICE_PNP_DEF_H

#define PNP_IDX_EN   0x30
#define PNP_IDX_IO0  0x60
#define PNP_IDX_IO1  0x62
#define PNP_IDX_IO2  0x64
#define PNP_IDX_IO3  0x66
#define PNP_IDX_IO4  0x68
#define PNP_IDX_IRQ0 0x70
#define PNP_IDX_IRQ1 0x72
#define PNP_IDX_DRQ0 0x74
#define PNP_IDX_DRQ1 0x75
#define PNP_IDX_MSC0 0xf0
#define PNP_IDX_MSC1 0xf1
#define PNP_IDX_MSC2 0xf2
#define PNP_IDX_MSC3 0xf3
#define PNP_IDX_MSC4 0xf4
#define PNP_IDX_MSC5 0xf5
#define PNP_IDX_MSC6 0xf6
#define PNP_IDX_MSC7 0xf7
#define PNP_IDX_MSC8 0xf8
#define PNP_IDX_MSC9 0xf9
#define PNP_IDX_MSCA 0xfa
#define PNP_IDX_MSCB 0xfb
#define PNP_IDX_MSCC 0xfc
#define PNP_IDX_MSCD 0xfd
#define PNP_IDX_MSCE 0xfe
#define PNP_IDX_RSVD 0xff

#endif /* DEVICE_PNP_DEF_H */
