/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _OPTION_IDS_H_
#define _OPTION_IDS_H_

#define IDSOPT_IDS_ENABLED			CONFIG(IDS_ENABLED)
#define IDSOPT_CONTROL_ENABLED			CONFIG(IDS_CONTROL_ENABLED)
#define IDSOPT_PERF_ANALYSIS			CONFIG(IDS_PERF_ANALYSIS)
#define IDSOPT_TRACING_ENABLED			CONFIG(IDS_TRACING_ENABLED)
#define IDSOPT_TRACING_CONSOLE_SERIALPORT	CONFIG(IDS_TRACING_CONSOLE_SERIAL)

#endif
