/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef TCG_TSS_COMMON_H_
#define TCG_TSS_COMMON_H_

#include <stdint.h>

#define TPM_PCR_MINIMUM_DIGEST_SIZE 20
#define TPM_SUCCESS ((uint32_t)0x00000000)

#endif /* TCG_TSS_COMMON_H_ */
