/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOUTHBRIDGE_RICOH_RL5C476
#define _SOUTHBRIDGE_RICOH_RL5C476

struct southbridge_ricoh_rl5c476_config {
	int enable_cf;
};

#endif /* _SOUTHBRIDGE_RICOH_RL5C476 */
