/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef SOUTHBRIDGE_INTEL_COMMON_FINALIZE_H
#define SOUTHBRIDGE_INTEL_COMMON_FINALIZE_H

void intel_pch_finalize_smm(void);

#endif /* SOUTHBRIDGE_INTEL_COMMON_FINALIZE_H */
