/*
 * Copyright 2017-present Facebook, Inc.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * bdk.h: This is a stub for BDK compatibility. The real bdk.h is an uber-
 * header that pulls in everything. For our purposes we'll create a minimal
 * version that includes only the stuff we need.
 */

#ifndef __SOC_CAVIUM_COMMON_BDK_H__
#define __SOC_CAVIUM_COMMON_BDK_H__

#include "bdk-minimal.h"

#endif /* !__SOC_CAVIUM_COMMON_BDK_H__ */
