/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef CPU_INTEL_CPU_IDS_H
#define CPU_INTEL_CPU_IDS_H

/* Supported CPUIDs for different Intel CPU */

#define CPUID_DENVERTON_A0_A1		0x506f0
#define CPUID_DENVERTON_B0		0x506f1
#define CPUID_COOPERLAKE_SP_A0		0x5065a
#define CPUID_COOPERLAKE_SP_A1		0x5065b
#define CPUID_SKYLAKE_SP_A0_A1		0x506f0
#define CPUID_SKYLAKE_SP_B0		0x506f1
#define CPUID_SKYLAKE_SP_4		0x50654
#define CPUID_SKYLAKE_C0		0x406e2
#define CPUID_SKYLAKE_D0		0x406e3
#define CPUID_SKYLAKE_HQ0		0x506e1
#define CPUID_SKYLAKE_HR0		0x506e3
#define CPUID_KABYLAKE_G0		0x406e8
#define CPUID_KABYLAKE_H0		0x806e9
#define CPUID_KABYLAKE_Y0		0x806ea
#define CPUID_KABYLAKE_HA0		0x506e8
#define CPUID_KABYLAKE_HB0		0x906e9
#define CPUID_CANNONLAKE_A0		0x60660
#define CPUID_CANNONLAKE_B0		0x60661
#define CPUID_CANNONLAKE_C0		0x60662
#define CPUID_CANNONLAKE_D0		0x60663
#define CPUID_APOLLOLAKE_A0		0x506c8
#define CPUID_APOLLOLAKE_B0		0x506c9
#define CPUID_APOLLOLAKE_E0		0x506ca
#define CPUID_GLK_A0			0x706a0
#define CPUID_GLK_B0			0x706a1
#define CPUID_GLK_R0			0x706a8
#define CPUID_WHISKEYLAKE_V0		0x806ec
#define CPUID_WHISKEYLAKE_W0		0x806eb
#define CPUID_COFFEELAKE_U0		0x906ea
#define CPUID_COFFEELAKE_B0		0x906eb
#define CPUID_COFFEELAKE_P0		0x906ec
#define CPUID_COFFEELAKE_R0		0x906ed
#define CPUID_ICELAKE_A0		0x706e0
#define CPUID_ICELAKE_B0		0x706e1
#define CPUID_JASPERLAKE_A0		0x906c0
#define CPUID_COMETLAKE_U_A0		0xa0660
#define CPUID_COMETLAKE_U_K0_S0		0xa0661
#define CPUID_COMETLAKE_H_S_6_2_G0	0xa0650
#define CPUID_COMETLAKE_H_S_6_2_G1	0xa0653
#define CPUID_COMETLAKE_H_S_10_2_P0	0xa0651
#define CPUID_COMETLAKE_H_S_10_2_Q0_P1	0xa0654
#define CPUID_TIGERLAKE_A0		0x806c0
#define CPUID_TIGERLAKE_B0		0x806c1
#define CPUID_TIGERLAKE_R0		0x806d1
#define CPUID_ELKHARTLAKE_A0		0x90660
#define CPUID_ELKHARTLAKE_B0		0x90661
#define CPUID_ALDERLAKE_S_A0		0x90670
#define CPUID_ALDERLAKE_A0		0x906a0
#define CPUID_ALDERLAKE_A1		0x906a1
#define CPUID_ALDERLAKE_A2		0x906a2
#define CPUID_ALDERLAKE_A3		0x906a4

#endif /* CPU_INTEL_CPU_IDS_H */
