/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __DRIVERS_INTEL_SOUNDWIRE_CHIP_H__
#define __DRIVERS_INTEL_SOUNDWIRE_CHIP_H__

struct drivers_intel_soundwire_config {
};

#endif /* __DRIVERS_INTEL_SOUNDWIRE_CHIP_H__ */
