/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_QUALCOMM_QCS405_SYMBOLS_H_
#define _SOC_QUALCOMM_QCS405_SYMBOLS_H_

#include <symbols.h>

DECLARE_REGION(ssram);
DECLARE_REGION(bsram);
DECLARE_REGION(dram_reserved);

#endif // _SOC_QUALCOMM_QCS405_SYMBOLS_H_
