/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_QUALCOMM_SC7280_UART_TX_H_
#define _SOC_QUALCOMM_SC7280_UART_TX_H_

#define UART_TX_PIN GPIO(22)

#endif /* _SOC_QUALCOMM_SC7280_UART_TX_H_ */
