/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* ISO C <ctype.h> like properties of Unicode characters.  */
/* Generated automatically by gen-uni-tables.c for Unicode 9.0.0.  */
#define header_0 16
#define header_2 9
#define header_3 127
#define header_4 15
static const
struct
  {
    int header[1];
    int level1[17];
    short level2[5 << 7];
    /*unsigned*/ int level3[78 << 4];
  }
u_is_print =
{
  { 17 },
  {
       18 * sizeof (int) / sizeof (short) +     0,
       18 * sizeof (int) / sizeof (short) +   128,
       18 * sizeof (int) / sizeof (short) +   256,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 * sizeof (int) / sizeof (short) +   384,
       18 * sizeof (int) / sizeof (short) +   512,
       18 * sizeof (int) / sizeof (short) +   512
  },
  {
       18 +   640 * sizeof (short) / sizeof (int) +     0,
       18 +   640 * sizeof (short) / sizeof (int) +    16,
       18 +   640 * sizeof (short) / sizeof (int) +    32,
       18 +   640 * sizeof (short) / sizeof (int) +    48,
       18 +   640 * sizeof (short) / sizeof (int) +    64,
       18 +   640 * sizeof (short) / sizeof (int) +    80,
       18 +   640 * sizeof (short) / sizeof (int) +    96,
       18 +   640 * sizeof (short) / sizeof (int) +   112,
       18 +   640 * sizeof (short) / sizeof (int) +   128,
       18 +   640 * sizeof (short) / sizeof (int) +   144,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   176,
       18 +   640 * sizeof (short) / sizeof (int) +   192,
       18 +   640 * sizeof (short) / sizeof (int) +   208,
       18 +   640 * sizeof (short) / sizeof (int) +   224,
       18 +   640 * sizeof (short) / sizeof (int) +   240,
       18 +   640 * sizeof (short) / sizeof (int) +   256,
       18 +   640 * sizeof (short) / sizeof (int) +   272,
       18 +   640 * sizeof (short) / sizeof (int) +   288,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   304,
       18 +   640 * sizeof (short) / sizeof (int) +   320,
       18 +   640 * sizeof (short) / sizeof (int) +   336,
       18 +   640 * sizeof (short) / sizeof (int) +   352,
       18 +   640 * sizeof (short) / sizeof (int) +   368,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   384,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   400,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   416,
       18 +   640 * sizeof (short) / sizeof (int) +   432,
       18 +   640 * sizeof (short) / sizeof (int) +   448,
       18 +   640 * sizeof (short) / sizeof (int) +   464,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   480,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   496,
       18 +   640 * sizeof (short) / sizeof (int) +   512,
       18 +   640 * sizeof (short) / sizeof (int) +   528,
       18 +   640 * sizeof (short) / sizeof (int) +   544,
       18 +   640 * sizeof (short) / sizeof (int) +   560,
       18 +   640 * sizeof (short) / sizeof (int) +   576,
       18 +   640 * sizeof (short) / sizeof (int) +   592,
       18 +   640 * sizeof (short) / sizeof (int) +   608,
       18 +   640 * sizeof (short) / sizeof (int) +   624,
       18 +   640 * sizeof (short) / sizeof (int) +   640,
       18 +   640 * sizeof (short) / sizeof (int) +   656,
       18 +   640 * sizeof (short) / sizeof (int) +   672,
       18 +   640 * sizeof (short) / sizeof (int) +   688,
       18 +   640 * sizeof (short) / sizeof (int) +   704,
       18 +   640 * sizeof (short) / sizeof (int) +   720,
       18 +   640 * sizeof (short) / sizeof (int) +   736,
       18 +   640 * sizeof (short) / sizeof (int) +   752,
       18 +   640 * sizeof (short) / sizeof (int) +   768,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   784,
       18 +   640 * sizeof (short) / sizeof (int) +   800,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   816,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   832,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   848,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   864,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   880,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   896,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   912,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   928,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   944,
       18 +   640 * sizeof (short) / sizeof (int) +   960,
       18 +   640 * sizeof (short) / sizeof (int) +   976,
       18 +   640 * sizeof (short) / sizeof (int) +   992,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1008,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +  1024,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +  1040,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +  1056,
       18 +   640 * sizeof (short) / sizeof (int) +  1072,
       18 +   640 * sizeof (short) / sizeof (int) +  1088,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1104,
       18 +   640 * sizeof (short) / sizeof (int) +  1120,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1136,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1152,
       18 +   640 * sizeof (short) / sizeof (int) +  1168,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1184,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1200,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +  1216,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +   160,
       18 +   640 * sizeof (short) / sizeof (int) +  1232
  },
  {
    0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF,
    0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFCFFFFFF,
    0xFFFFD7F0, 0xFFFFFFFB, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFEFFFF, 0xFE7FFFFF, 0xFFFFFFFE,
    0xFFFEE6FF, 0xFFFFFFFF, 0xFFFF00FF, 0x001F07FF,
    0xDFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFBFFF, 0xFFFFFFFF, 0xFFFFE7FF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x0003FFFF, 0xFFFFFFFF, 0x07FFFFFF,
    0xFFFFFFFF, 0x7FFF3FFF, 0x4FFFFFFF, 0x00000000,
    0x00000000, 0x3FDFFFFF, 0xFFF00000, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFF99FEF, 0xF3C5FDFF, 0xB080799F, 0x0FFFFFCF,
    0xFFF987EE, 0xD36DFDFF, 0x5E023987, 0x003FFFC0,
    0xFFFBBFEE, 0xF3EDFDFF, 0x00013BBF, 0x0203FFCF,
    0xFFF99FEE, 0xF3EDFDFF, 0xB0C0399F, 0x00FFFFCF,
    0xD63DC7EC, 0xC3FFC718, 0x00813DC7, 0x07FFFFC0,
    0xFFFDDFEF, 0xE3FFFDFF, 0x07603DDF, 0xFF00FFCF,
    0xFFFDDFEF, 0xF3EFFDFF, 0x40603DDF, 0x0006FFCF,
    0xFFFDDFEE, 0xE7FFFFFF, 0xFFF0FDDF, 0xFFFFFFCF,
    0xFC7FFFEC, 0x2FFBFFFF, 0xFF5F847F, 0x001CFFC0,
    0xFFFFFFFE, 0x87FFFFFF, 0x0FFFFFFF, 0x00000000,
    0xFEF02596, 0x3BFFECAE, 0xF3FF3F5F, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFE1FFF,
    0xFEFFFFFF, 0xDFFFFFFF, 0x07FFDFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF20BF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x3D7F3DFF, 0xFFFFFFFF,
    0xFFFF3DFF, 0x7F3DFFFF, 0xFF7FFF3D, 0xFFFFFFFF,
    0xFF3DFFFF, 0xFFFFFFFF, 0xE7FFFFFF, 0x1FFFFFFF,
    0x03FFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x3F3FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x1FFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x01FFFFFF,
    0x001FDFFF, 0x007FFFFF, 0x000FFFFF, 0x000DDFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x3FFFFFFF, 0x03FF03FF,
    0x03FF7FFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00FFFFFF,
    0xFFFFFFFF, 0xFFFF07FF, 0xFFFFFFFF, 0x003FFFFF,
    0x7FFFFFFF, 0x0FFF0FFF, 0xFFFFFFF1, 0x001F3FFF,
    0xFFFFFFFF, 0xFFFF0FFF, 0xC7FF03FF, 0xFFFFFFFF,
    0xCFFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF, 0x9FFFFFFF,
    0x03FF03FF, 0x7FFF3FFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF0FFF, 0x1FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF00FFFFF,
    0xFFFFFFFF, 0xF8FFFFFF, 0xFFFFE3FF, 0xFFFFFFFF,
    0x000001FF, 0x00000000, 0xFFFF00FF, 0x037FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF83FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3F3FFFFF, 0xFFFFFFFF, 0xAAFF3F3F, 0x3FFFFFFF,
    0xFFFFFFFF, 0xFFDFFFFF, 0xEFCFFFDF, 0x7FDCFFFF,
    0xFFFFFFFF, 0xFFFFFCFF, 0xFFFFFFFF, 0xFFF3FFDF,
    0x1FFF7FFF, 0x7FFFFFFF, 0xFFFF0000, 0x0001FFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFF0FFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF,
    0xFFFFFFFF, 0x0000007F, 0x000007FF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFFFFF,
    0xFF3FFFFF, 0xE3FFFFFF, 0x0003FDFF, 0x0000F000,
    0xFFFFFFFF, 0xFFFF7FFF, 0x7FFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFE0FFFFF,
    0xFFFFFFFF, 0xFFFF20BF, 0xFFFFFFFF, 0x800180FF,
    0x007FFFFF, 0x7F7F7F7F, 0x7F7F7F7F, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000001F, 0x00000000,
    0xFBFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x003FFFFF, 0x0FFF0000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFF,
    0xFE7FFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFE0, 0xFFFE3FFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFF7FFF, 0x07FFFFFF, 0xFFFFFFFF, 0xFFFF000F,
    0x7FFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x003FFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x003FFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFF1FFF, 0xFFFFFFFF, 0xFFFF007F, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x00000FFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00FFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x00FF7FFF, 0x00000000, 0xFF800000,
    0xFFFFFFFF, 0x03FF0FFF, 0xFFFFFFFF, 0x00FFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FFC03F, 0x3FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x800FFFFF, 0x1FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xC3FFBFFF, 0x7FFFFFFF,
    0xFFFFFFFF, 0x007FFFFF, 0xF3FF3FFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xF8000007, 0x007FFFFF,
    0x007E7E7E, 0xFFFF7F7F, 0xFFFFFFFF, 0xFFFF003F,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x03FF3FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFF000F, 0xFFFFF87F, 0x0FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF3FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FFFFFF, 0x00000000,
    0xE0F8007F, 0x5F7FFFFF, 0xFFFFFFDB, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFF80003, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF0000, 0xFFFFFFFF,
    0xFFFCFFFF, 0xFFFFFFFF, 0x000000FF, 0x3FFF0000,
    0x03FFFFFF, 0xFFFFFFFF, 0xFFF7FFFF, 0xFFDF0F7F,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x9FFFFFFF,
    0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x7FFFFFFF, 0x1CFCFCFC, 0x3E007F7F,
    0xFFFFEFFF, 0xB7FFFF7F, 0x3FFF3FFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x07FFFFFF,
    0xFFFFFF87, 0xFF8FFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x0FFF7FFF, 0x00000001, 0xFFFF0000, 0x3FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x1FFFFFFF, 0xFFFFFFFF, 0x0001FFFF, 0x0FFFFFFF,
    0xFFFFFFFF, 0xFFFF000F, 0xFFFF07FF, 0x07FFFFFF,
    0xBFFFFFFF, 0xFFFFFFFF, 0x003FFF0F, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0xFFFF03FF, 0xFF0FFFFF, 0x0FFFFFFF,
    0xFFFFFFFF, 0xFFFF00FF, 0xFFFFFFFF, 0x0000800F,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x007FFFFF, 0x003FFFFF, 0x000000FF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFD3F, 0x91BFFFFF, 0xFFBFFFFF, 0xFFFFFFFF,
    0x7FFFFFFF, 0x0000FF80, 0x00000000, 0xF837FFFF,
    0x8FFFFFFF, 0x83FFFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xF0FFFFFF, 0xFFFCFFFF, 0xFFFFFFFF,
    0xFEEFF06F, 0x870FFFFF, 0x01FF00FF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x007FF87F,
    0xFFFFFFFF, 0xFE3FFFFF, 0xFF3FFFFF, 0xFF07FFFF,
    0x1E03FFFF, 0x0000FE00, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x000001FF, 0x00000000,
    0xFFFFFFFF, 0x0007FFFF, 0xFFFFFFFF, 0xFC07FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x7FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFC3FFF, 0x8000FFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF0003, 0x03FF01FF,
    0xFFFFFFFF, 0xFFDFFFFF, 0xFFFF000F, 0x007FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF3FFF, 0x001FFFFE,
    0xFFFBFFFF, 0x7FFFFFFF, 0x00000000, 0x00000000,
    0xBFFFBD7F, 0xFFFF03FF, 0xFFFFFFFF, 0x03FF07FF,
    0xFFF99FEF, 0xF3EDFDFF, 0xE081399F, 0x001F1FCF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x2BFFFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FF00FF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFF3FFFFF, 0x3FFFFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FF001F, 0x00001FFF,
    0xFFFFFFFF, 0x00FFFFFF, 0x000003FF, 0x00000000,
    0xE3FFFFFF, 0xFFFF0FFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x8007FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0xFFFFFFFF, 0x01FFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFDFF, 0xFF7FFFFF, 0xFFFF003F, 0xFFFF1FFF,
    0xFFFCFFFF, 0x007FFEFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x03FFFFFF, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001F7FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000000F, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x00007FFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000007F, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x01FFFFFF, 0x7FFFFFFF, 0x0000C3FF,
    0x00000000, 0x00000000, 0xFFFF0000, 0x003F3FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFBFF003F, 0xE0FFFFFB,
    0x0000FFFF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF001F, 0x7FFFFFFF,
    0xFFFF8000, 0x00000000, 0x00000000, 0x00000001,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00001FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0007FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000003, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x1FFF07FF,
    0xF3FF01FF, 0x0000000F, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x003FFFFF,
    0xFFFFFFFF, 0xFFFFFE7F, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000001FF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000003F, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x007FFFFF, 0x0003FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFDFFFFF, 0xFFFFFFFF,
    0xDFFFFFFF, 0xEBFFDE64, 0xFFFFFFEF, 0xFFFFFFFF,
    0xDFDFE7BF, 0x7BFFFFFF, 0xFFFDFC5F, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFF3F, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFCFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xF8000FFF, 0x0000FFFE, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xF9FFFF7F, 0x000007DB, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x007FFF9F, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xC3FF07FF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFEF, 0x0AF7FE96, 0xAA96EA84, 0x5EF7F796,
    0x0FFFFBFF, 0x0FFFFBEE, 0x00000000, 0x00030000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFF0FFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x000FFFFF, 0xFFFE7FFF, 0xFFFEFFFE, 0x003FFFFF,
    0xFFFF1FFF, 0xFFFF7FFF, 0xFFFFFFFF, 0xFFFF0FFF,
    0xFFFFFFFF, 0x00001FFF, 0x00000000, 0xFFFFFFC0,
    0xFFFF0007, 0x0FFFFFFF, 0x000301FF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0007FFFF, 0x007F1FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x001FFFFF, 0x00000000,
    0xFFFF0FFF, 0xFFFFFFFF, 0x03FF00FF, 0xFFFFFFFF,
    0xFFFF00FF, 0x00003FFF, 0x00000000, 0x00000000,
    0x7FFF0000, 0x7FF900FF, 0x7FFF0FFF, 0x00000000,
    0x0003FFFF, 0x00000000, 0x00000001, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x007FFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x001FFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x00000003, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x3FFFFFFF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000002, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0000FFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x3FFFFFFF
  }
};
