/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_SMI_H
#define SOC_SMI_H

#define SMI_SCI_TRIG			0x08
#define SMI_SCI_LEVEL			0x0c
#define SMI_SCI_STATUS			0x10
#define SMI_SCI_EN			0x14

#endif /* SOC_SMI_H */
