/* SPDX-License-Identifier: GPL-2.0-only */

#include <bootmem.h>
#include <boot/tables.h>
#include <boot/coreboot_tables.h>

void arch_write_tables(uintptr_t coreboot_table)
{
}

void bootmem_arch_add_ranges(void)
{
}

void lb_arch_add_records(struct lb_header *header)
{
}
