/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __I2C_QCOM_HEADER___
#define __I2C_QCOM_HEADER___

#include <device/i2c.h>

void i2c_init(unsigned int bus, enum i2c_speed speed);

#endif /* __I2C_QCOM_HEADER */
