/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_GPIO_H
#define SOC_GPIO_H

/* <soc/gpio.h> must provide gpio_t. */
#include <amdblocks/gpio.h>

#define SOC_GPIO_TOTAL_PINS		133

#endif /* SOC_GPIO_H */
